
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#include "StdAfx.h"
#include "Font.h"
#include "Range.h"
#include <autowrap.h>

/*
	CXLFont class represents Font Excel object. It contains the font attributes 
	(font name, font size, color, and so on) for an object.

*/
CXLFont::CXLFont(): m_pRange(NULL)
{
}

CXLFont::~CXLFont()
{
    if (m_pRange != NULL)
    {
        m_pRange->Release();
    }
}

HRESULT CXLFont::Initialize(CRange *pRange)
{
    if (m_pRange != NULL)
    {
        m_pRange->Release();
    }
    m_pRange = pRange;
    if (m_pRange != NULL)
    {
        m_pRange->AddRef();
    }
    return S_OK;
}

STDMETHODIMP CXLFont::get_Bold(VARIANT_BOOL *pvbBold)
{
    WRITE_LOG_A(("CXLFont::get_Bold(...)"));

    if (pvbBold == NULL)
    {
        return E_POINTER;
    }
// In OO bold is specified as weight of the character

	VARIANT vBoldState;
	VariantInit (&vBoldState);
	HRESULT hres = AutoWrap(DISPATCH_PROPERTYGET, &vBoldState,
		m_pRange->m_pdRange, L"CharWeight", 0);
	if (hres != S_OK)  {
		*pvbBold = VARIANT_FALSE;
		return hres;
	}
	if (vBoldState.fltVal > 140)
		*pvbBold = VARIANT_TRUE;
	else 
		*pvbBold = VARIANT_FALSE;
    return S_OK;
}

STDMETHODIMP CXLFont::put_Bold(VARIANT_BOOL vbBold)
{
    WRITE_LOG_A(("CXLFont::put_Bold(%d)", vbBold));

	VARIANT vBoldState;
	VariantInit (&vBoldState);
	vBoldState.vt = VT_R4;
	if (vbBold == VARIANT_TRUE)
		vBoldState.fltVal = 150;
	else
		vBoldState.fltVal = 100;
	VARIANT res;
	HRESULT hres = AutoWrap(DISPATCH_PROPERTYPUT, &res, m_pRange->m_pdRange, L"CharWeight", 1, vBoldState);
	return hres;
}

STDMETHODIMP CXLFont::get_Italic(VARIANT_BOOL *pvbItalic)
{
    WRITE_LOG_A(("CXLFont::get_Italic(...)"));

    if (pvbItalic == NULL)
    {
        return E_POINTER;
    }
	VARIANT vItalicState;
	VariantInit (&vItalicState);
	HRESULT hres = AutoWrap(DISPATCH_PROPERTYGET, &vItalicState, m_pRange->m_pdRange, L"CharPosture", 0);
	if (hres != S_OK)
		return hres;
	if (vItalicState.intVal != 0)
		*pvbItalic = VARIANT_TRUE;
	else
		*pvbItalic = VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CXLFont::put_Italic(VARIANT_BOOL vbItalic)
{
    WRITE_LOG_A(("CXLFont::put_Italic(%d)", vbItalic));
	
	VARIANT vItalicState;
	VariantInit (&vItalicState);
	vItalicState.vt = VT_INT;
	if (vbItalic == VARIANT_TRUE)
		vItalicState.intVal = 3;
	else
		vItalicState.intVal = 0;
	VARIANT res;
	HRESULT hres = AutoWrap(DISPATCH_PROPERTYPUT, &res, m_pRange->m_pdRange, L"CharPosture", 1, vItalicState);
    return S_OK;
}

STDMETHODIMP CXLFont::get_Underline(VARIANT_BOOL *pvbUnderline)
{
    WRITE_LOG_A(("CXLFont::get_Underline(...)"));
	
    if (pvbUnderline == NULL)
    {
        return E_POINTER;
    }
	VARIANT vUnderlineState;
	VariantInit (&vUnderlineState);
	HRESULT hres = AutoWrap(DISPATCH_PROPERTYGET, &vUnderlineState, m_pRange->m_pdRange, L"CharUnderline", 0);
	if (hres != S_OK)
		return hres;
	*pvbUnderline = vUnderlineState.boolVal;
    return S_OK;
}

STDMETHODIMP CXLFont::put_Underline(VARIANT_BOOL vbUnderline)
{
    WRITE_LOG_A(("CXLFont::put_Underline(%d)", vbUnderline));
	VARIANT vUnderlineState;
	VariantInit (&vUnderlineState);
	vUnderlineState.vt = VT_BOOL;
	vUnderlineState.intVal = vbUnderline;
	VARIANT res;
	HRESULT hres = AutoWrap(DISPATCH_PROPERTYPUT, &res, m_pRange->m_pdRange, L"CharUnderline", 1, vUnderlineState);
    return S_OK;
}
