
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#include "StdAfx.h"
#include "Application.h"
#include "Workbook.h"
#include "Worksheet.h"
#include "Range.h"

#include <autowrap.h>
/*
	CWorksheet class represents Excel worksheet object by using OpenOffice.
	It implements IWorksheet COM interface.
*/
CWorksheet::CWorksheet():
	m_allCells(this), 
    m_pdSheet(NULL),
    m_pParentWorkbook(NULL)
{
    WRITE_LOG_R_W((L"CWorksheet::CWorksheet()"));
	m_allCells.Initialize(m_pdSheet);
    //m_AllCells.Initialize(NULL, CELL_COORD(1, 1), CELL_COORD(0, 0));
}

CWorksheet::~CWorksheet()
{
    WRITE_LOG_R_W((L"CWorksheet::~CWorksheet()"));
}

STDMETHODIMP CWorksheet::get_Name(BSTR *pbstrName)
{
    WRITE_LOG_R_W((L"CWorksheet::get_Name(...)"));

	if (pbstrName == NULL) {
        return E_POINTER;
    }
	
	VARIANT res;
	VariantInit(&res);
	HRESULT hres = AutoWrap(DISPATCH_METHOD, &res, m_pdSheet, L"getName", 0);
	//Auto
	if (hres == S_OK)
		*pbstrName = res.bstrVal;
    return hres;
}

STDMETHODIMP CWorksheet::put_Name(BSTR bstrName)
{
    WRITE_LOG_R_W((L"CWorksheet::put_Name(\"%s\")", (bstrName == NULL ? L"" : bstrName)));


	VARIANT res;
	VariantInit(&res);
	VARIANT new_str;
	VariantInit(&new_str);
	new_str.vt = VT_BSTR;
	new_str.bstrVal = bstrName;
	HRESULT hres = AutoWrap(DISPATCH_METHOD, &res, m_pdSheet, L"setName", 1, new_str);

    return hres;
}

STDMETHODIMP CWorksheet::get_Cells(IRange **ppRange)
{
    WRITE_LOG_R_W((L"CWorksheet::get_Cells(...)"));

    if (ppRange == NULL)
    {
        return E_POINTER;
    }
    *ppRange = &m_allCells;
    m_allCells.AddRef();
    return S_OK;
}

void CWorksheet::Initialize(CWorkbook *pParentWorkbook, IDispatch *pdSheet)
{
    WRITE_LOG_R_W((L"CWorksheet::Initialize"));
    m_pParentWorkbook = pParentWorkbook;
	m_pdSheet = pdSheet;
	m_allCells.Initialize(this->m_pdSheet);//, CELL_COORD(1, 1), CELL_COORD(3000, 3000)
}

// NEW!!!
STDMETHODIMP CWorksheet::get_Range    (VARIANT Cell1, VARIANT Cell2, IRange **ppRange)
{
    WRITE_LOG_R_W((L"CWorksheet::get_Range(...)"));
    CRange *pCell1 = /*dynamic_cast<CRange*> */ (CRange*) (Cell1.pdispVal);
    CRange *pCell2 = /*dynamic_cast<CRange*> */ (CRange*) (Cell2.pdispVal);
    if ((pCell1 == NULL) || (pCell2 == NULL))
    {
        return E_INVALIDARG;
    }
    HRESULT hres;
    long lCell1L, lCell1R, lCell1T, lCell1B;
    long lCell2L, lCell2R, lCell2T, lCell2B;
//    long lRangeL, lRangeR, lRangeT, lRangeB;
    hres = pCell1->GetRangeAddress(lCell1L, lCell1T, lCell1R, lCell1B);
    if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FAIL"));
        return hres;
    }
    hres = pCell2->GetRangeAddress(lCell2L, lCell2T, lCell2R, lCell2B);
    if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FAIL"));
        return hres;
    }

    CRangeIntObject *pRange = new CRangeIntObject(this);
    if (pRange == NULL)
    {
        return E_FAIL;
    }
    hres = pRange->Initialize(&m_allCells, CELL_COORD(lCell1L + 1, lCell1T + 1), CELL_COORD(lCell2R + 1, lCell2B + 1));
    if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FAIL"));
        delete pRange;
        return hres;
    }
    pRange->AddRef();
    *ppRange = pRange;
    return S_OK;
}
// NEW!!!
STDMETHODIMP CWorksheet::Paste   (VARIANT Destination, VARIANT Link, long lcid)
{

    WRITE_LOG_R_W((L"CWorksheet::Paste(...)"));
    if ((CApplication::s_pdOOApp == NULL) || (Destination.vt != VT_DISPATCH))
    {
        return E_FAIL;
    }
    //CF_TEXT
    HRESULT hres;
    ::OleInitialize(NULL);
    IDataObject *pDataObj;
    hres = ::OleGetClipboard(&pDataObj);
    if (hres != S_OK)
    {
        return hres;
    }
	FORMATETC		fmtetc;
	STGMEDIUM		stgmed = {0};
    fmtetc.cfFormat = CF_TEXT;
	fmtetc.dwAspect = DVASPECT_CONTENT;
	fmtetc.lindex	= -1;
	fmtetc.ptd		= 0;
	fmtetc.tymed	= TYMED_HGLOBAL;

    hres = pDataObj->GetData(&fmtetc, &stgmed);
    pDataObj->Release();
    if (hres != S_OK)
    {
        pDataObj->Release();
        return hres;
    }

    char* szPasteString = (char*) GlobalLock(stgmed.hGlobal);
    ::GlobalUnlock(stgmed.hGlobal);
    if (szPasteString == NULL)
    {
        return E_FAIL;
    }
    
    //VARIANT vRes; 

    CRange *pRange = (CRange*) Destination.pdispVal;
    CComVariant vStr = szPasteString;
    CComVariant vAnyValue;
    hres = pRange->put_Value(vAnyValue, vStr);
    return hres;
}
